import { __decorate, __awaiter, __param } from 'tslib';
import { ElementRef, HostBinding, Input, Output, Component, forwardRef, EventEmitter, ɵɵdefineInjectable, ɵɵinject, Injectable, Attribute, TemplateRef, ViewChild, ViewContainerRef, ComponentFactoryResolver, ApplicationRef, Injector, INJECTOR, Directive, Self, Renderer2, HostListener, InjectionToken, Inject, Optional, Host, ContentChildren, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR, NgForm, NgModel, NgControl, FormsModule } from '@angular/forms';
import { BehaviorSubject, of, Subject, merge, fromEvent } from 'rxjs';
import { take, finalize, filter, debounceTime, switchMap, takeUntil, map, tap, distinctUntilChanged, takeWhile } from 'rxjs/operators';
import { NavigationEnd, ActivatedRoute, Router, RouterModule } from '@angular/router';
import { isEqual, escapeRegExp } from 'lodash';
import { CommonModule } from '@angular/common';
import { DomSanitizer } from '@angular/platform-browser';

var ImageInputComponent_1;
class ImageInputValue {
}
let ImageInputComponent = ImageInputComponent_1 = class ImageInputComponent {
    constructor(elementRef) {
        this.elementRef = elementRef;
        this.randomId = btoa(Math.random().toString());
        this.bootstrap = true;
        this.draggedOver = false;
        this.disabled = false;
        this.interactive = true;
        this.bgImage = '';
        this.valueChange = new BehaviorSubject(this.value);
        this.onChangeFn = (x) => null;
    }
    get hasImage() {
        return !!this.value;
    }
    set placeholderValue(value) {
        this.originalValue = value;
        if (value && value.src) {
            this.bgImage = `url(${value.src})`;
        }
        else {
            this.bgImage = '';
        }
    }
    set value(value) {
        if (this.disabled) {
            return;
        }
        if (this.modifier) {
            this.disabled = true;
            of(value)
                .pipe(this.modifier, take(1), finalize(() => {
                this.disabled = false;
            }))
                .subscribe(v => {
                this.disabled = false;
                this.originalValue = v;
                this.onChangeFn(v);
                this.valueChange.next(v);
                this.bgImage = `url(${v.src})`;
            });
            return;
        }
        this.originalValue = value;
        this.onChangeFn(value);
        this.valueChange.next(value);
        this.bgImage = `url(${value.src})`;
    }
    get value() {
        return this.originalValue;
    }
    ngOnInit() {
    }
    registerOnChange(fn) {
        this.onChangeFn = fn;
    }
    registerOnTouched(fn) {
    }
    writeValue(obj) {
        if (this.disabled || !obj) {
            return;
        }
        this.originalValue = obj;
        this.bgImage = `url(${obj.src})`;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
    fileChange($event) {
        const target = ($event.target);
        const files = (target.files || $event.dataTransfer.files);
        if (files.length) {
            const reader = new FileReader();
            reader.onload = () => {
                const value = new ImageInputValue();
                value.blob = files.item(0);
                value.src = reader.result;
                value.id = btoa(Math.random().toString());
                this.value = value;
                target.value = null;
            };
            reader.readAsDataURL(files.item(0));
        }
        else {
            // this.hasImage = false;
            // this.value = {};
        }
    }
};
ImageInputComponent.ctorParameters = () => [
    { type: ElementRef }
];
__decorate([
    HostBinding('class.form-control')
], ImageInputComponent.prototype, "bootstrap", void 0);
__decorate([
    HostBinding('class.dragging-over')
], ImageInputComponent.prototype, "draggedOver", void 0);
__decorate([
    HostBinding('class.has-image')
], ImageInputComponent.prototype, "hasImage", null);
__decorate([
    HostBinding('class.disabled'), Input()
], ImageInputComponent.prototype, "disabled", void 0);
__decorate([
    HostBinding('class.interactive'), Input()
], ImageInputComponent.prototype, "interactive", void 0);
__decorate([
    Input()
], ImageInputComponent.prototype, "placeholderValue", null);
__decorate([
    Input()
], ImageInputComponent.prototype, "value", null);
__decorate([
    Output()
], ImageInputComponent.prototype, "valueChange", void 0);
__decorate([
    Input()
], ImageInputComponent.prototype, "modifier", void 0);
ImageInputComponent = ImageInputComponent_1 = __decorate([
    Component({
        // tslint:disable-next-line:component-selector
        selector: 'image-input',
        template: "<div class=\"image\" [style.backgroundImage]=\"bgImage\"></div>\n<input type=\"file\"\n       accept=\"image/png, image/jpeg\"\n       (change)=\"fileChange($event)\"\n       (dragenter)=\"draggedOver = true\"\n       (dragover)=\"draggedOver = true\"\n       (dragend)=\"draggedOver = false\"\n       (dragleave)=\"draggedOver = false\"\n       (drop)=\"draggedOver = false; fileChange($event)\"\n       [id]=\"randomId\">\n<label [for]=\"randomId\">\n  <svg aria-hidden=\"true\" focusable=\"false\" data-prefix=\"fas\" data-icon=\"file-upload\" role=\"img\" xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 384 512\" class=\"svg-inline--fa fa-file-upload fa-w-12 fa-3x\"><path fill=\"#909090\" d=\"M224 136V0H24C10.7 0 0 10.7 0 24v464c0 13.3 10.7 24 24 24h336c13.3 0 24-10.7 24-24V160H248c-13.2 0-24-10.8-24-24zm65.18 216.01H224v80c0 8.84-7.16 16-16 16h-32c-8.84 0-16-7.16-16-16v-80H94.82c-14.28 0-21.41-17.29-11.27-27.36l96.42-95.7c6.65-6.61 17.39-6.61 24.04 0l96.42 95.7c10.15 10.07 3.03 27.36-11.25 27.36zM377 105L279.1 7c-4.5-4.5-10.6-7-17-7H256v128h128v-6.1c0-6.3-2.5-12.4-7-16.9z\" class=\"\"></path></svg>\n</label>\n",
        providers: [{
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => ImageInputComponent_1),
                multi: true
            }],
        styles: [":host{min-width:200px;min-height:200px;position:relative;display:flex;padding:0;background-color:rgba(0,0,0,.1)}:host .image{background-size:contain;background-repeat:no-repeat;background-position:center center;height:100%!important;width:100%!important;flex:1;position:absolute}:host input[type=file]{height:100%!important;width:100%!important;border:0!important;margin:0!important;padding:0!important;position:absolute;flex:1;opacity:0;-webkit-filter:opacity(0);filter:opacity(0);cursor:pointer}:host label{display:flex;flex:1;max-height:100%;flex-direction:column;position:relative;margin:0!important;pointer-events:none}:host label svg{flex:1;height:0;min-height:50px;margin:auto}:host.dragging-over label{background:rgba(0,0,0,.2)}:host.has-image label{-webkit-filter:opacity(0);filter:opacity(0)}:host.has-image label svg{-webkit-filter:brightness(130%);filter:brightness(130%)}:host.has-image.dragging-over label,:host.has-image:hover label{-webkit-filter:opacity(1);filter:opacity(1);background:rgba(0,0,0,.2);display:flex}:host.disabled input,:host.disabled label{display:none!important;pointer-events:none!important}:host:not(.interactive) input,:host:not(.interactive) label{pointer-events:none;display:none}"]
    })
], ImageInputComponent);

var ImageGalleryComponent_1;
class GalleryImageValue extends ImageInputValue {
}
let ImageGalleryComponent = ImageGalleryComponent_1 = class ImageGalleryComponent {
    constructor() {
        this.disabled = false;
        this.bootstrap = true;
        this.valueChange = new BehaviorSubject(this.value);
        this.delete = new EventEmitter();
        this.randomId = btoa(Math.random().toString());
        this.index = -1;
        this.originalValue = [
        // {
        //   src: 'https://source.unsplash.com/random',
        //   sort: 0
        // },
        // {
        //   src: 'https://source.unsplash.com/random',
        //   sort: 1
        // },
        // {
        //   src: 'https://source.unsplash.com/random',
        //   sort: 2
        // },
        // {
        //   src: 'https://source.unsplash.com/random',
        //   sort: 2
        // },
        ];
        this.dir = 'ltr';
        this.onChangeFn = (x) => null;
        const targetNode = window.document.documentElement;
        const config = { attributes: true, childList: false, subtree: false };
        const callback = (mutationsList, observer) => {
            // Use traditional 'for loops' for IE 11
            for (const mutation of mutationsList) {
                if (mutation.type === 'attributes') {
                    this.dir = targetNode.getAttribute('dir') || 'ltr';
                }
            }
        };
        this.htmlElementObserver = new MutationObserver(callback);
        this.htmlElementObserver.observe(targetNode, config);
    }
    set value(value) {
        if (this.disabled) {
            return;
        }
        this.originalValue = value.sort((a, b) => a.sort - b.sort);
        this.onChangeFn(value);
        this.valueChange.next(value);
        if (!this.selected && value.length) {
            this.selected = this.originalValue[0];
        }
    }
    get value() {
        return this.originalValue;
    }
    ngOnDestroy() {
        this.htmlElementObserver.disconnect();
    }
    registerOnChange(fn) {
        this.onChangeFn = fn;
    }
    registerOnTouched(fn) {
    }
    writeValue(obj) {
        if (!obj) {
            return;
        }
        this.originalValue = obj.sort((a, b) => a.sort - b.sort);
        if (!this.selected && obj.length) {
            this.selected = this.originalValue[0];
        }
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
    newImage($event) {
        if (!$event) {
            return;
        }
        const newImage = Object.assign(Object.assign({}, $event), { sort: this.value.length });
        this.originalValue.push(newImage);
        this.value = this.originalValue;
        this.selected = newImage;
    }
    deleteImage(index) {
        this.delete.next({ index, images: this.originalValue, deleted: this.originalValue[index] });
        delete this.originalValue[index];
        for (let i = index; i < this.originalValue.length - 1; i++) {
            this.originalValue[i] = this.originalValue[i + 1];
            this.originalValue[i].sort = i;
        }
        if (this.value.length) {
            if (index > 0) {
                this.selected = this.originalValue[index];
            }
            else {
                this.selected = this.originalValue[index - 1];
            }
        }
        else {
            this.selected = null;
        }
        this.originalValue.splice(this.originalValue.length - 1, 1);
        this.value = this.originalValue;
    }
    swapImages(aIndex, bIndex) {
        if (!this.originalValue[aIndex] || !this.originalValue[bIndex]) {
            return;
        }
        const temp = this.originalValue[aIndex];
        this.originalValue[aIndex] = this.originalValue[bIndex];
        this.originalValue[bIndex] = temp;
        for (let i = 0; i < this.originalValue.length; i++) {
            this.originalValue[i].sort = i;
        }
    }
};
__decorate([
    HostBinding('class.disabled'), Input()
], ImageGalleryComponent.prototype, "disabled", void 0);
__decorate([
    HostBinding('class.form-control')
], ImageGalleryComponent.prototype, "bootstrap", void 0);
__decorate([
    Output()
], ImageGalleryComponent.prototype, "valueChange", void 0);
__decorate([
    Output()
], ImageGalleryComponent.prototype, "delete", void 0);
__decorate([
    Input()
], ImageGalleryComponent.prototype, "imageInputModifier", void 0);
ImageGalleryComponent = ImageGalleryComponent_1 = __decorate([
    Component({
        // tslint:disable-next-line:component-selector
        selector: 'image-gallery',
        template: "<div class=\"d-flex\">\n  <ng-container *ngIf=\"value.length > 1\">\n    <button type=\"button\" class=\"btn btn-light\" (click)=\"swapImages(selected.sort, selected.sort - 1)\" *ngIf=\"!this.disabled\">\n      <i\n        class=\"fas fa-fw fa-chevron-left\"\n        *ngIf=\"dir === 'rtl'\"></i>\n      <i\n        class=\"fas fa-fw fa-chevron-right\"\n        *ngIf=\"dir === 'ltr'\"></i>\n    </button>\n    <span class=\"mx-1 my-auto\">\n    {{selected?.sort + 1}}\n      <small> / {{value.length}}</small>\n    </span>\n    <button type=\"button\" class=\"btn btn-light\" (click)=\"swapImages(selected.sort, selected.sort + 1)\" *ngIf=\"!this.disabled\">\n      <i\n        class=\"fas fa-fw fa-chevron-right\"\n        *ngIf=\"dir === 'rtl'\"></i>\n      <i\n        class=\"fas fa-fw fa-chevron-left\"\n        *ngIf=\"dir === 'ltr'\"></i>\n    </button>\n  </ng-container>\n  <div class=\"flex-grow-1\"></div>\n  <button type=\"button\"\n          (click)=\"deleteImage(selected.sort)\"\n          class=\"btn btn-danger\" *ngIf=\"value.length && !this.disabled\">\n    <i class=\"fas fa-fw fa-trash\"></i>\n  </button>\n</div>\n<image-input\n  [placeholderValue]=\"selected\"\n  [disabled]=\"disabled\"\n  [modifier]=\"imageInputModifier\"\n  (valueChange)=\"newImage($event)\"\n  class=\"flex-grow-1 mt-1\"\n></image-input>\n<div class=\"row flex-nowrap gallery-slider p-md-4 p-2 mx-n2 \" *ngIf=\"value.length > 1\">\n  <div class=\"col-md-4 slide-item d-flex flex-column px-1\" *ngFor=\"let image of value; let i = index\">\n    <div class=\"bg-white rounded position-relative flex-grow-1 clickable shadow-sm\"\n         [class.shadow]=\"image.id === selected.id\"\n         (click)=\"selected = image\">\n      <div class=\"image\" [style.backgroundImage]=\"'url(' + image.src + ')'\"></div>\n    </div>\n  </div>\n</div>\n",
        providers: [{
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => ImageGalleryComponent_1),
                multi: true
            }],
        styles: [":host{min-width:200px;min-height:300px;position:relative;display:flex;flex-direction:column}:host .gallery-slider{overflow-x:auto}:host .gallery-slider .slide-item{min-height:100px;min-width:100px}:host .gallery-slider .slide-item .image{background-size:contain;background-repeat:no-repeat;background-position:center center;height:100%!important;width:100%!important;flex-grow:1;position:absolute}"]
    })
], ImageGalleryComponent);

let BreadcrumbsService = class BreadcrumbsService {
    constructor(activatedRoute, router) {
        this.activatedRoute = activatedRoute;
        this.router = router;
        this.breadcrumbs$ = new BehaviorSubject([]);
        this.refresh$ = new Subject();
        this.firstRun = true;
        this.previousUrl = '';
        merge(router.events
            .pipe(filter(event => event instanceof NavigationEnd), debounceTime(200)), this.refresh$).subscribe((event) => __awaiter(this, void 0, void 0, function* () {
            const url = this.router.url.replace(/\?.*/, '');
            if (event && !this.firstRun) {
                if (url === this.previousUrl) {
                    return;
                }
            }
            this.previousUrl = url;
            this.breadcrumbs$.next(yield this.digBreadcrumbs(activatedRoute.root.snapshot, !event));
            this.firstRun = false;
        }));
    }
    digBreadcrumbs(route, patched = false, breadcrumbs = []) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const config = (route.routeConfig || {});
            const breadcrumb = {
                id: btoa(Math.random().toString()),
                url: '/' + route.pathFromRoot.map(o => o.url[0]).filter(u => !!(u === null || u === void 0 ? void 0 : u.path)).join('/'),
                index: breadcrumbs.length,
            };
            // @ts-ignore
            (_a = route.routeConfig) === null || _a === void 0 ? void 0 : _a.id = breadcrumb.id;
            if (patched && (config === null || config === void 0 ? void 0 : config.breadcrumbPatched)) {
                breadcrumb.label = config.breadcrumbPatched;
            }
            else if ((config === null || config === void 0 ? void 0 : config.breadcrumb) || ((_b = config === null || config === void 0 ? void 0 : config.data) === null || _b === void 0 ? void 0 : _b.breadcrumb)) {
                delete config.breadcrumbPatched;
                breadcrumb.label = (config === null || config === void 0 ? void 0 : config.breadcrumb) || ((_c = config === null || config === void 0 ? void 0 : config.data) === null || _c === void 0 ? void 0 : _c.breadcrumb);
            }
            breadcrumbs.push(breadcrumb);
            const child = route.children.find(r => r.outlet === 'primary');
            if (child) {
                return this.digBreadcrumbs(child, patched, breadcrumbs);
            }
            return breadcrumbs;
        });
    }
    updateBreadcrumb(route, label) {
        const config = route.routeConfig;
        config.breadcrumbPatched = label;
        this.refresh$.next(null);
    }
};
BreadcrumbsService.ctorParameters = () => [
    { type: ActivatedRoute },
    { type: Router }
];
BreadcrumbsService.ɵprov = ɵɵdefineInjectable({ factory: function BreadcrumbsService_Factory() { return new BreadcrumbsService(ɵɵinject(ActivatedRoute), ɵɵinject(Router)); }, token: BreadcrumbsService, providedIn: "any" });
BreadcrumbsService = __decorate([
    Injectable({
        providedIn: 'any',
    })
], BreadcrumbsService);

let BreadcrumbsComponent = class BreadcrumbsComponent {
    constructor(breadcrumbsService) {
        this.breadcrumbsService = breadcrumbsService;
        this.breadcrumbListClassValue = '';
        this.breadcrumbListItemClassValue = '';
    }
    set breadcrumbListClass(breadcrumbListClass) {
        this.breadcrumbListClassValue = breadcrumbListClass;
    }
    get breadcrumbListClass() {
        return this.breadcrumbListClassValue;
    }
    set breadcrumbListItemClass(breadcrumbListItemClass) {
        this.breadcrumbListItemClassValue = breadcrumbListItemClass;
    }
    get breadcrumbListItemClass() {
        return this.breadcrumbListItemClassValue;
    }
    ngOnInit() {
    }
};
BreadcrumbsComponent.ctorParameters = () => [
    { type: BreadcrumbsService }
];
__decorate([
    Input()
], BreadcrumbsComponent.prototype, "linkTemplate", void 0);
__decorate([
    Attribute('breadcrumbListClass'), Input()
], BreadcrumbsComponent.prototype, "breadcrumbListClass", null);
__decorate([
    Attribute('breadcrumbListItemClass'), Input()
], BreadcrumbsComponent.prototype, "breadcrumbListItemClass", null);
BreadcrumbsComponent = __decorate([
    Component({
        // tslint:disable-next-line:component-selector
        selector: 'nav[breadcrumbs]',
        template: "<ol class=\"breadcrumb {{breadcrumbListClass}}\">\n  <ng-container *ngFor=\"let breadcrumb of breadcrumbsService.breadcrumbs$ | async\">\n    <li class=\"breadcrumb-item {{breadcrumbListItemClass}}\" routerLinkActive=\"active\" *ngIf=\"breadcrumb.label\">\n      <ng-container *ngTemplateOutlet=\"linkTemplate || templateRef; context: breadcrumb\"></ng-container>\n    </li>\n  </ng-container>\n</ol>\n\n<ng-template #templateRef let-label=\"label\" let-url=\"url\" let-index=\"index\">\n  <a [routerLink]=\"url\" routerLinkActive=\"active\" [routerLinkActiveOptions]=\"{exact: true}\">{{label}}</a>\n</ng-template>\n",
        styles: [""]
    })
], BreadcrumbsComponent);

let PaginationComponent = class PaginationComponent {
    constructor(router, route) {
        this.router = router;
        this.route = route;
        this.pages$ = new BehaviorSubject([]);
        this.page$ = new BehaviorSubject(1);
        this.refreshQuery$ = new BehaviorSubject(null);
        this.queryCheckedFirst = false;
        this.pageChange = new EventEmitter();
        this.ngDestroy$ = new EventEmitter();
    }
    set pageSize(pageSize) {
        this.pageSizeValue = pageSize;
        this.updatePages();
    }
    get pageSize() {
        return this.pageSizeValue;
    }
    set collectionSize(collectionSize) {
        this.collectionSizeValue = collectionSize;
        this.updatePages();
    }
    get collectionSize() {
        return this.collectionSizeValue;
    }
    set maxSize(maxSize) {
        this.maxSizeValue = maxSize;
        this.updatePages();
    }
    get maxSize() {
        return this.maxSizeValue;
    }
    set page(page) {
        this.pageValue = page;
        this.page$.next(+page);
        this.updatePages();
    }
    get page() {
        return +this.pageValue || 1;
    }
    set pagination(pagination) {
        Object.assign(this, pagination);
        this.updatePages();
    }
    set queryParamValue(queryParam) {
        this.queryParam = [true, ''].includes(queryParam) ? 'page' : queryParam === false ? '' : queryParam;
        this.refreshQuery$.next(null);
        this.page = this.page;
        this.updatePages();
    }
    updatePages() {
        if (!(this.collectionSize && this.pageSize)) {
            return;
        }
        let pages = Math.ceil(+this.collectionSize / +this.pageSize);
        this.lastPage = pages;
        let base = 1;
        if (pages > this.maxSize) {
            base = Math.max(1, Math.min(Math.round(+this.page - (+this.maxSize * 0.5)), pages - +this.maxSize + 1));
            pages = +this.maxSize;
        }
        this.pages$.next(new Array(Math.ceil(pages)).fill(base).map((a, i) => i + a));
    }
    ngOnInit() {
        this.refreshQuery$
            .pipe(switchMap(() => this.route.queryParams), takeUntil(this.ngDestroy$), filter(() => !!this.queryParam), map(params => params[this.queryParam]), tap(page => this.queryCheckedFirst = isNaN(page)), filter(page => !isNaN(+page)), distinctUntilChanged(), tap(page => {
            this.page = page;
            this.queryCheckedFirst = true;
        }))
            .subscribe();
        this.page$
            .pipe(takeUntil(this.ngDestroy$), filter(page => !isNaN(+page)), distinctUntilChanged(), tap(page => this.pageChange.emit(page)), filter(() => this.queryParam && this.queryCheckedFirst), tap(page => {
            const queryParams = {};
            queryParams[this.queryParam] = page;
            this.router.navigate([], { queryParams, queryParamsHandling: 'merge' });
        }))
            .subscribe();
    }
    ngOnDestroy() {
        this.ngDestroy$.next();
    }
};
PaginationComponent.ctorParameters = () => [
    { type: Router },
    { type: ActivatedRoute }
];
__decorate([
    Output()
], PaginationComponent.prototype, "pageChange", void 0);
__decorate([
    Attribute('pageSize'), Input()
], PaginationComponent.prototype, "pageSize", null);
__decorate([
    Attribute('collectionSize'), Input()
], PaginationComponent.prototype, "collectionSize", null);
__decorate([
    Attribute('maxSize'), Input()
], PaginationComponent.prototype, "maxSize", null);
__decorate([
    Attribute('page'), Input()
], PaginationComponent.prototype, "page", null);
__decorate([
    Input()
], PaginationComponent.prototype, "pagination", null);
__decorate([
    Attribute('queryParam'), Input('queryParam')
], PaginationComponent.prototype, "queryParamValue", null);
PaginationComponent = __decorate([
    Component({
        // tslint:disable-next-line:component-selector
        selector: 'nav[pagination]',
        template: "<ul class=\"pagination\">\n  <li class=\"page-item\" [class.disabled]=\"page <= 1\">\n    <button class=\"page-link\" aria-label=\"Previous\" (click)=\"page = 1\" [disabled]=\"page <= 1\">\n      <span aria-hidden=\"true\">&laquo;&laquo;</span>\n      <span class=\"sr-only\">First</span>\n    </button>\n  </li>\n  <li class=\"page-item\" [class.disabled]=\"page <= 1\">\n    <button class=\"page-link\" aria-label=\"Previous\" (click)=\"page = page - 1\" [disabled]=\"page <= 1\">\n      <span aria-hidden=\"true\">&laquo;</span>\n      <span class=\"sr-only\">Previous</span>\n    </button>\n  </li>\n  <ng-container *ngFor=\"let p of pages$ | async\">\n    <li class=\"page-item\" [class.active]=\"+p === page\">\n      <button class=\"page-link\" (click)=\"page = p\">{{p}}</button>\n    </li>\n  </ng-container>\n  <li class=\"page-item\" [class.disabled]=\"page >= lastPage\">\n    <button class=\"page-link\" aria-label=\"Previous\" (click)=\"page = page + 1\" [disabled]=\"page >= lastPage\">\n      <span aria-hidden=\"true\">&raquo;</span>\n      <span class=\"sr-only\">Next</span>\n    </button>\n  </li>\n  <li class=\"page-item\" [class.disabled]=\"page >= lastPage\">\n    <button class=\"page-link\" aria-label=\"Previous\" (click)=\"page = lastPage\" [disabled]=\"page >= lastPage\">\n      <span aria-hidden=\"true\">&raquo;&raquo;</span>\n      <span class=\"sr-only\">Last</span>\n    </button>\n  </li>\n</ul>\n",
        styles: [""]
    })
], PaginationComponent);

const popoverStyleElement = `
  <style>
    popover-container {
        position: fixed;
        top: 0;
        left: 0;
        z-index: 2000;
    }

    popover-container flying-popover {
        position: absolute;
        left: 0;
        top: 0;
        z-index: 0;
        background-color: #fff;
        will-change: transform;
        filter: opacity(0);
        pointer-events: none;
        border-radius: 0.25rem;
        border-color: white;
    }

    popover-container flying-popover.show {
        filter: opacity(1) drop-shadow(0 0 0.5rem rgba(0, 0, 0, 0.3));
        pointer-events: auto;
    }

    popover-container flying-popover::before {
        content: '';
        display: block;
        border-color: inherit;
        position: absolute;
        z-index: -1;
    }

    popover-container flying-popover.arrow.top-left {
        border-bottom-right-radius: 0;
        margin-top: -.5rem;
    }

    popover-container flying-popover.arrow.top-left::before {
        top: calc(100%);
        left: calc(100% - .5rem);
        border-bottom: .5rem solid transparent;
        border-right: .5rem solid;
        border-right-color: inherit;
    }

    popover-container flying-popover.arrow.bottom-left {
        border-top-right-radius: 0;
        margin-top: .5rem;
    }

    popover-container flying-popover.arrow.bottom-left::before {
        bottom: calc(100%);
        left: calc(100% - .5rem);
        border-top: .5rem solid transparent;
        border-right: .5rem solid;
        border-right-color: inherit;
    }

    popover-container flying-popover.arrow.middle-left {
        margin-left: -.5rem;
    }

    popover-container flying-popover.arrow.middle-left::before {
        top: calc(50% - 0.25rem);
        left: calc(100%);
        border-top: .25rem solid transparent;
        border-bottom: .25rem solid transparent;
        border-left: .5rem solid;
        border-left-color: inherit;
    }

    popover-container flying-popover.arrow.top-right {
        border-bottom-left-radius: 0;
        margin-top: -.5rem;
    }

    popover-container flying-popover.arrow.top-right::before {
        top: calc(100%);
        right: calc(100% - .5rem);
        border-bottom: .5rem solid transparent;
        border-left: .5rem solid;
        border-left-color: inherit;
    }

    popover-container flying-popover.arrow.bottom-right {
          border-top-left-radius: 0;
          margin-top: .5rem;
    }

    popover-container flying-popover.arrow.bottom-right::before {
        bottom: calc(100%);
        right: calc(100% - .5rem);
        border-top: .5rem solid transparent;
        border-left: .5rem solid;
        border-left-color: inherit;
    }

    popover-container flying-popover.arrow.middle-right {
        margin-left: .5rem;
    }

    popover-container flying-popover.arrow.middle-right::before {
        top: calc(50% - 0.25rem);
        right: calc(100%);
        border-top: .25rem solid transparent;
        border-bottom: .25rem solid transparent;
        border-right: .5rem solid;
        border-right-color: inherit;
    }

    popover-container flying-popover.arrow.bottom-center {
        margin-top: .5rem;
    }

    popover-container flying-popover.arrow.bottom-center::before {
        bottom: calc(100%);
        left: calc(50% - .25rem);
        border-left: .25rem solid transparent;
        border-right: .25rem solid transparent;
        border-bottom: .5rem solid;
        border-bottom-color: inherit;
    }

    popover-container flying-popover.arrow.top-center {
        margin-top: -.5rem;
    }

    popover-container flying-popover.arrow.top-center::before {
        top: calc(100%);
        left: calc(50% - .25rem);
        border-left: .25rem solid transparent;
        border-right: .25rem solid transparent;
        border-top: .5rem solid;
        border-top-color: inherit;
    }

    popover-container flying-popover .popover-body {
        padding: .5rem;
        width: max-content;
    }
  </style>
`;
class Popover {
    constructor({ content, anchorElement, horizontal, vertical, element, popoverComponentRef, }) {
        this.arrowClassValue = 'arrow';
        this.arrowClass = this.arrowClassValue;
        this.showValue = false;
        this.forceShow = false;
        this.customClass = '';
        this.matchWidth = false;
        this.ready = new Subject();
        if (element) {
            this.element = element;
        }
        else if (popoverComponentRef) {
            this.popoverComponentRef = popoverComponentRef;
            this.element = popoverComponentRef.hostView.rootNodes[0];
        }
        else {
            this.element = document.createElement('flying-popover');
        }
        this.id = btoa(Math.random().toString());
        this.anchorElement = anchorElement;
        this.content = content;
        this.horizontal = horizontal || this.horizontal;
        this.vertical = vertical || this.vertical;
        this.ready.next(this);
        this.ready = of(this);
    }
    set show(showValue) {
        this.showValue = showValue;
        this.updatePopOverPosition();
    }
    get show() {
        return this.showValue || this.forceShow;
    }
    set arrow(b) {
        this.arrowClass = b ? this.arrowClassValue : '';
    }
    set content(content) {
        if (typeof content === 'string') {
            this.element.innerHTML = `<div class="popover-body">${content}</div>`;
        }
        else if (content instanceof HTMLElement) {
            this.element.innerHTML = ``;
            this.element.appendChild(content);
        }
        else if (Array.isArray(content)) {
            this.element.innerHTML = ``;
            content.forEach(c => {
                if (c instanceof HTMLElement) {
                    this.element.appendChild(c);
                }
            });
        }
        else if (content instanceof TemplateRef && this.popoverComponentRef) {
            this.popoverComponentRef.instance.setContent(content);
        }
    }
    getPosition() {
        const anchorRect = this.anchorElement.getBoundingClientRect();
        if (this.matchWidth === 'min-width') {
            this.element.style.minWidth = anchorRect.width + 'px';
        }
        else if (this.matchWidth) {
            this.element.style.width = anchorRect.width + 'px';
        }
        else {
            delete this.element.style.width;
        }
        if (this.maxWindowHeight) {
            this.element.style.maxHeight = `calc(70vh - ${anchorRect.height}px - 2em)`;
            this.element.style.overflowY = 'auto';
        }
        else {
            this.element.style.maxHeight = ``;
            this.element.style.overflowY = '';
        }
        const popoverRect = this.element.getBoundingClientRect();
        let horizontal = this.horizontal;
        let vertical = this.vertical;
        let top = anchorRect.top;
        let left = anchorRect.left;
        const rtl = document.documentElement.getAttribute('dir') === 'rtl';
        switch (horizontal) {
            case 'center':
                left += (anchorRect.width * .5) - (popoverRect.width * .5);
                break;
            case 'start':
                left += rtl ? anchorRect.width + popoverRect.width : -popoverRect.width;
                horizontal = rtl ? 'right' : 'left';
                break;
            case 'end':
                left += rtl ? -popoverRect.width : anchorRect.width;
                horizontal = rtl ? 'left' : 'right';
                break;
            case 'left':
                left += -popoverRect.width;
                break;
            case 'right':
                left += anchorRect.width;
                break;
            case 'left-baseline':
                left += 0;
                horizontal = 'right';
                break;
            case 'right-baseline':
                left += anchorRect.width - popoverRect.width;
                horizontal = 'left';
                break;
            case 'start-baseline':
                left += rtl ? anchorRect.width - popoverRect.width : 0;
                horizontal = rtl ? 'left' : 'right';
                break;
            case 'end-baseline':
                left += rtl ? 0 : anchorRect.width - popoverRect.width;
                horizontal = rtl ? 'right' : 'left';
                break;
            case 'auto':
                if (!rtl) {
                    if (left + anchorRect.width >= window.innerWidth) {
                        left += -popoverRect.width;
                        horizontal = 'left';
                    }
                    else {
                        left += anchorRect.width;
                        horizontal = 'right';
                    }
                }
                else {
                    if (left < 0) {
                        left += anchorRect.width;
                        horizontal = 'right';
                    }
                    else {
                        left += -popoverRect.width;
                        horizontal = 'left';
                    }
                }
                break;
        }
        left = Math.max(0, Math.min(window.innerWidth - popoverRect.width, left));
        left = left + 'px';
        switch (vertical) {
            case 'middle':
                top += (anchorRect.height * .5) - (popoverRect.height * .5);
                break;
            case 'top':
                top += -popoverRect.height;
                break;
            case 'bottom':
                top += anchorRect.height;
                break;
            case 'top-baseline':
                top += 0;
                break;
            case 'bottom-baseline':
                top += anchorRect.height - popoverRect.height;
                break;
            case 'auto':
                if (top + anchorRect.height + popoverRect.height > window.innerHeight) {
                    top += -popoverRect.height;
                    vertical = 'top';
                }
                else {
                    top += anchorRect.height;
                    vertical = 'bottom';
                }
                break;
        }
        top = Math.max(0, Math.min(window.innerHeight - popoverRect.height, top));
        top = top + 'px';
        return {
            top,
            left,
            horizontal,
            vertical,
        };
    }
    updatePopOverPosition() {
        const position = this.getPosition();
        requestAnimationFrame(() => {
            this.element.style.transform = `translate(${position.left}, ${position.top})`;
            this.element.className = this.show ?
                `show ${position.vertical}-${position.horizontal} ${this.customClass} ${this.arrowClass}`
                : this.customClass;
        });
    }
}
let FlyingPopoverComponent = class FlyingPopoverComponent {
    constructor() {
        this.content$ = new BehaviorSubject(null);
        this.ngDestroy$ = new Subject();
    }
    setContent(template) {
        this.content$.next(template);
    }
    ngAfterContentInit() {
        this.content$
            .pipe(takeUntil(this.ngDestroy$), filter(template => template))
            .subscribe((template) => {
            this.container.clear();
            this.container.createEmbeddedView(template);
        });
    }
    ngOnDestroy() {
        this.ngDestroy$.next();
    }
};
__decorate([
    ViewChild('container', { static: true, read: ViewContainerRef })
], FlyingPopoverComponent.prototype, "container", void 0);
FlyingPopoverComponent = __decorate([
    Component({
        // tslint:disable-next-line:component-selector
        selector: 'flying-popover',
        template: '<ng-template #container></ng-template>',
        styles: ['']
    })
], FlyingPopoverComponent);
let PopoverService = class PopoverService {
    constructor(componentFactoryResolver, appRef, injector) {
        this.componentFactoryResolver = componentFactoryResolver;
        this.appRef = appRef;
        this.injector = injector;
        this.popovers = [];
        this.underObservation = [];
        this.containerElement = document.createElement('popover-container');
        this.containerElement.innerHTML = (popoverStyleElement);
        document.body.append(this.containerElement);
        merge(fromEvent(window, 'scroll', { capture: true }), fromEvent(window, 'resize')
            .pipe(debounceTime(1000)))
            .subscribe(() => {
            this.underObservation.forEach((popover) => popover.updatePopOverPosition());
        });
    }
    addPopover(popover) {
        return __awaiter(this, void 0, void 0, function* () {
            const popoverComponentRef = this.componentFactoryResolver.resolveComponentFactory(FlyingPopoverComponent).create(this.injector);
            this.appRef.attachView(popoverComponentRef.hostView);
            delete popover.element;
            popover.popoverComponentRef = popoverComponentRef;
            const service = this;
            const newPopover = new Proxy(new Popover(popover), {
                set(target, p, value) {
                    if (p === 'show') {
                        target[p] = value;
                        if (!target.show) {
                            const index = service.underObservation.findIndex(pop => pop.id === target.id);
                            if (index > -1) {
                                service.underObservation.splice(index, 1);
                            }
                        }
                        else {
                            const index = service.underObservation.findIndex(pop => pop.id === target.id);
                            if (index === -1) {
                                service.underObservation.push(target);
                            }
                        }
                    }
                    target[p] = value;
                    return true;
                }
            });
            this.containerElement.append(popoverComponentRef.hostView.rootNodes[0]);
            this.popovers.push(newPopover);
            if (!newPopover.show) {
                this.underObservation.push(newPopover);
            }
            newPopover.updatePopOverPosition();
            return (newPopover);
        });
    }
    removePopover(id) {
        const index = this.popovers.findIndex(pop => pop.id === id);
        const index2 = this.underObservation.findIndex(pop => pop.id === id);
        if (index !== -1) {
            this.popovers[index].element.remove();
            delete this.popovers[index];
            this.popovers.splice(index, 1);
        }
        if (index2 !== -1) {
            delete this.underObservation[index];
            this.underObservation.splice(index, 1);
        }
    }
};
PopoverService.ctorParameters = () => [
    { type: ComponentFactoryResolver },
    { type: ApplicationRef },
    { type: Injector }
];
PopoverService.ɵprov = ɵɵdefineInjectable({ factory: function PopoverService_Factory() { return new PopoverService(ɵɵinject(ComponentFactoryResolver), ɵɵinject(ApplicationRef), ɵɵinject(INJECTOR)); }, token: PopoverService, providedIn: "root" });
PopoverService = __decorate([
    Injectable({
        providedIn: 'root'
    })
], PopoverService);

let PopoverDirective = class PopoverDirective {
    constructor(popoverService, elementRef) {
        this.popoverService = popoverService;
        this.elementRef = elementRef;
        this.ngDestroy$ = new Subject();
        this.popoverReady = new EventEmitter();
        this.popoverContentValue$ = new BehaviorSubject('');
        this.popoverOnHook = '';
        this.popoverPlacementValue = {
            horizontal: 'auto',
            vertical: 'auto',
        };
        this.popoverClassValue = '';
        this.popoverArrowValue = true;
        this.popoverMatchWidthValue = false;
        this.popoverMaxWindowHeightValue = false;
    }
    set popoverContent(content) {
        this.popoverContentValue$.next(content);
    }
    set popoverOn(hook) {
        if (hook === this.popoverOnHook) {
            return;
        }
        this.popoverOnHook = hook;
        this.updateHook();
    }
    set popoverPlacement(popoverPlacementValue) {
        var _a;
        this.popoverPlacementValue = popoverPlacementValue;
        (_a = this.popover) === null || _a === void 0 ? void 0 : _a.updatePopOverPosition();
    }
    set popoverClass(customClass) {
        this.popoverClassValue = customClass;
        if (this.popover) {
            this.popover.customClass = customClass;
        }
    }
    set popoverArrow(customArrow) {
        this.popoverArrowValue = customArrow;
        if (this.popover) {
            this.popover.arrow = customArrow;
        }
    }
    set popoverMatchWidth(matchWidth) {
        this.popoverMatchWidthValue = matchWidth;
        if (this.popover) {
            this.popover.matchWidth = matchWidth;
        }
    }
    set popoverMatchHeight(matchHeight) {
        this.popoverMaxWindowHeightValue = matchHeight;
        if (this.popover) {
            this.popover.maxWindowHeight = matchHeight;
        }
    }
    show() {
        this.popover.show = true;
    }
    hide() {
        this.popover.show = false;
    }
    forceShow(bool = true) {
        this.popover.forceShow = bool;
        this.popover.updatePopOverPosition();
    }
    ngAfterContentInit() {
        return __awaiter(this, void 0, void 0, function* () {
            this.popover = yield this.popoverService.addPopover({
                anchorElement: this.elementRef.nativeElement,
                content: this.popoverContentValue$.value,
                vertical: this.popoverPlacementValue.vertical,
                horizontal: this.popoverPlacementValue.horizontal,
            });
            this.popover.customClass = this.popoverClassValue;
            this.popover.arrow = this.popoverArrowValue;
            this.popover.matchWidth = this.popoverMatchWidthValue;
            this.popover.maxWindowHeight = this.popoverMaxWindowHeightValue;
            setTimeout(() => this.popover.updatePopOverPosition(), 1000);
            this.updateHook();
            this.popoverContentValue$
                .pipe(takeUntil(this.ngDestroy$), tap(elements => this.popover.content = elements))
                .subscribe();
            this.popoverReady.emit(this);
        });
    }
    updateHook() {
        if (!this.popover) {
            return;
        }
        switch (this.popoverOnHook) {
            case 'hover':
                this.activateOnHover();
                break;
            case 'click':
                this.activateOnClick();
                break;
        }
    }
    activateOnHover() {
        fromEvent(this.popover.element, 'mouseover', { capture: true })
            .pipe(takeUntil(this.ngDestroy$), takeWhile(() => this.popoverOnHook === 'hover'), tap(() => this.popover.forceShow = true))
            .subscribe();
        fromEvent(this.popover.element, 'mouseleave')
            .pipe(takeUntil(this.ngDestroy$), takeWhile(() => this.popoverOnHook === 'hover'), tap(() => this.popover.forceShow = false), tap(() => this.popover.updatePopOverPosition()))
            .subscribe();
        fromEvent(this.elementRef.nativeElement, 'mouseover', { capture: true })
            .pipe(takeUntil(this.ngDestroy$), takeWhile(() => this.popoverOnHook === 'hover'))
            .subscribe(() => this.popover.show = true);
        fromEvent(this.elementRef.nativeElement, 'mouseleave')
            .pipe(takeUntil(this.ngDestroy$), takeWhile(() => this.popoverOnHook === 'hover'))
            .subscribe(() => this.popover.show = false);
        fromEvent(window, 'click')
            .pipe(takeUntil(this.ngDestroy$), takeWhile(() => this.popoverOnHook === 'hover'))
            .subscribe((event) => !this.popover.element.contains(event.target) ?
            this.popover.show = this.popover.forceShow = false
            : null);
    }
    activateOnClick() {
        fromEvent(this.elementRef.nativeElement, 'click')
            .pipe(takeUntil(this.ngDestroy$), takeWhile(() => this.popoverOnHook === 'click'))
            .subscribe(() => this.popover.show = !this.popover.show);
        fromEvent(window, 'click')
            .pipe(takeUntil(this.ngDestroy$), takeWhile(() => this.popoverOnHook === 'click'))
            .subscribe((event) => !this.popover.element.contains(event.target)
            &&
                !this.elementRef.nativeElement.contains(event.target)
            ? this.popover.show = this.popover.forceShow = false
            : null);
    }
    ngOnDestroy() {
        var _a;
        this.ngDestroy$.next();
        this.popoverService.removePopover((_a = this.popover) === null || _a === void 0 ? void 0 : _a.id);
    }
};
PopoverDirective.ctorParameters = () => [
    { type: PopoverService },
    { type: ElementRef }
];
__decorate([
    Output()
], PopoverDirective.prototype, "popoverReady", void 0);
__decorate([
    Input()
], PopoverDirective.prototype, "popoverContent", null);
__decorate([
    Input()
], PopoverDirective.prototype, "popoverOn", null);
__decorate([
    Input()
], PopoverDirective.prototype, "popoverPlacement", null);
__decorate([
    Input()
], PopoverDirective.prototype, "popoverClass", null);
__decorate([
    Input()
], PopoverDirective.prototype, "popoverArrow", null);
__decorate([
    Input()
], PopoverDirective.prototype, "popoverMatchWidth", null);
__decorate([
    Input()
], PopoverDirective.prototype, "popoverMatchHeight", null);
PopoverDirective = __decorate([
    Directive({
        // tslint:disable-next-line:directive-selector
        selector: '[popover]',
        exportAs: 'popover'
    })
], PopoverDirective);

let NgFormQueryParamsDirective = class NgFormQueryParamsDirective {
    constructor(form, route, router) {
        this.form = form;
        this.route = route;
        this.router = router;
        this.ngDestroy$ = new EventEmitter();
        this.refreshQuery$ = new BehaviorSubject(null);
        this.queryCheckedFirst = false;
        this.queryParams = {};
    }
    ngOnDestroy() {
        this.ngDestroy$.next();
    }
    ngAfterContentInit() {
        this.refreshQuery$
            .pipe(switchMap(() => this.route.queryParams), takeUntil(this.ngDestroy$), distinctUntilChanged((pre, cur) => {
            return isEqual(pre, cur);
        }), tap(queryParams => {
            var _a;
            this.queryParams = queryParams;
            for (const p of Object.keys(queryParams)) {
                const value = queryParams[p];
                (_a = this.form.form.get(p)) === null || _a === void 0 ? void 0 : _a.setValue(value);
            }
        }))
            .subscribe();
        this.form.valueChanges
            .pipe(takeUntil(this.ngDestroy$), distinctUntilChanged((pre, cur) => {
            return isEqual(pre, cur);
        }), debounceTime(500), tap(queryParams => {
            var _a;
            if (!this.queryCheckedFirst) {
                for (const p of Object.keys(this.queryParams)) {
                    const value = this.queryParams[p];
                    (_a = this.form.form.get(p)) === null || _a === void 0 ? void 0 : _a.setValue(value);
                }
                this.queryCheckedFirst = true;
                return;
            }
            this.router.navigate([], { queryParams, queryParamsHandling: 'merge' });
        }))
            .subscribe();
    }
};
NgFormQueryParamsDirective.ctorParameters = () => [
    { type: NgForm, decorators: [{ type: Self }] },
    { type: ActivatedRoute },
    { type: Router }
];
NgFormQueryParamsDirective = __decorate([
    Directive({
        // tslint:disable-next-line:directive-selector
        selector: 'form[queryParams]'
    }),
    __param(0, Self())
], NgFormQueryParamsDirective);

class PressAndHoldEvent extends Event {
    constructor() {
        super('pressandhold');
    }
}
function pressAndHoldEventInit(element, delay = 500) {
    const ev = new PressAndHoldEvent();
    let frame;
    const pressDown = () => {
        element.dispatchEvent(ev);
        const time = performance.now();
        const pressHold = () => {
            if (time + delay <= performance.now()) {
                element.dispatchEvent(ev);
            }
            frame = requestAnimationFrame(pressHold);
        };
        frame = requestAnimationFrame(pressHold);
    };
    const pressUp = () => {
        cancelAnimationFrame(frame);
    };
    element.addEventListener('touchstart', pressDown, false);
    element.addEventListener('touchend', pressUp, false);
    element.addEventListener('touchcancel', pressUp, false);
    element.addEventListener('mousedown', pressDown, false);
    element.addEventListener('mouseup', pressUp, false);
    element.addEventListener('mouseleave', pressUp, false);
}

const numberRegex = /(?:[-+]?\d+\.?\d*)|[\s-+*/]/g;
let AdvanceInputDirective = class AdvanceInputDirective {
    constructor(model, elementRef, renderer) {
        this.model = model;
        this.elementRef = elementRef;
        this.renderer = renderer;
        this.ngDestroy$ = new Subject();
        this.prefixValue = '';
        this.suffixValue = '';
        this.value = '';
        this.cursorPosition = 0;
        this.model.valueAccessor.registerOnChange = function registerOnChange(fn) {
            this.onChange = fn;
        };
    }
    set type(type) {
        this.typeValue = type;
        if (this.element) {
            this.element.type = 'text';
        }
    }
    set prefix(prefix) {
        this.prefixValue = prefix;
    }
    set suffix(suffix) {
        this.suffixValue = suffix;
    }
    set controls(controls) {
        var _a, _b;
        this.controlsValue = controls;
        if (this.typeValue === 'number') {
            if (this.controlsValue && this.element) {
                this.initializeControls();
            }
            else {
                (_a = this.incrementElement) === null || _a === void 0 ? void 0 : _a.remove();
                (_b = this.decrementElement) === null || _b === void 0 ? void 0 : _b.remove();
            }
        }
    }
    set min(min) {
        this.minValue = min;
    }
    get min() {
        return this.minValue;
    }
    set max(max) {
        this.maxValue = max;
    }
    get max() {
        return this.maxValue;
    }
    set template(template) {
        this.templateValue = template;
    }
    get template() {
        return this.templateValue;
    }
    set pattern(pattern) {
        this.patternValue = pattern;
    }
    get pattern() {
        return this.patternValue;
    }
    updateValue(value = '') {
        value = (value + '').replace(new RegExp('^' + escapeRegExp(this.prefixValue)), '');
        value = (value + '').replace(new RegExp(escapeRegExp(this.suffixValue) + '$'), '');
        if (this.backspace && value.length === 0) {
            this.backspace = false;
            return this.updateModel(value, '');
        }
        const viewValue = this.updateView(value);
        const modelValue = this.updateModel(value, viewValue);
        this.value = modelValue;
        return modelValue;
    }
    updateView(originalValue) {
        var _a, _b;
        let value = originalValue || '';
        let formattedValue;
        let indexFormat;
        let indexNumber;
        let selection;
        let brackets;
        let subLength = 0;
        let cursorEnd = false;
        // @ts-ignore
        if (document.selection) {
            // @ts-ignore
            selection = document.selection.createRange();
            selection.moveStart('character', -value.length);
            this.cursorPosition = selection.text.length;
        }
        else if (this.element.selectionStart || this.element.selectionStart === 0) {
            this.cursorPosition = this.element.selectionStart;
        }
        switch (this.typeValue) {
            case 'number':
                value = value.replace(/(\s)+/g, '$1');
                value = value.replace(/([^-+*/]|\s)([-+*/])+/g, '$1$2');
                value = value.replace(/(\s[-+*/]\s)[-+*/]/g, '$1');
                // remove the repetition of the left edge zeros of numbers
                value = value.replace(/^([\s]*)(0)+/g, '$1$2');
                value = value.replace(/([^\d.])(0)+/g, '$1$2');
                // remove the left edge zeros of numbers
                value = value.replace(/^([\s]*)(?:0|(0\.))([\d]+)/g, '$1$2$3');
                value = value.replace(/([^\d.])(?:0|(0\.))([\d]+)/g, '$1$2$3');
                value = (_a = value.match(numberRegex)) === null || _a === void 0 ? void 0 : _a.join('');
                if (!/\.$/.test(value)) {
                    const decimals = (_b = (value + '').split('.')[1]) === null || _b === void 0 ? void 0 : _b.split('').filter(f => !isNaN(+f)).length;
                    value = !isNaN(this.min) && !isNaN(+value) ? Math.max(+value, this.min).toFixed(decimals) + '' : value;
                    value = !isNaN(this.max) && !isNaN(+value) ? Math.min(+value, this.max).toFixed(decimals) + '' : value;
                }
                break;
            case 'tel':
                if (this.template) {
                    if (this.cursorPosition === value.length) {
                        cursorEnd = true;
                    }
                    if (this.backspace && this.template.charAt(value.length) !== 'd') {
                        value = (String(value).substring(0, value.length - 1));
                    }
                    formattedValue = '';
                    const num = String(value).replace(/\D/g, '');
                    for (indexFormat = 0, indexNumber = 0; indexFormat < this.template.length; indexFormat = indexFormat + 1) {
                        if (/\d/g.test(this.template.charAt(indexFormat))) {
                            if (this.template.charAt(indexFormat) === num.charAt(indexNumber)) {
                                formattedValue += num.charAt(indexNumber);
                                indexNumber = indexNumber + 1;
                            }
                            else {
                                formattedValue += this.template.charAt(indexFormat);
                            }
                        }
                        else if (this.template.charAt(indexFormat) === '[' && this.template.charAt(indexFormat + 1) === '[') {
                            brackets = this.template.substr(indexFormat + 2);
                            brackets = brackets.substr(0, brackets.indexOf(']'));
                            if (!brackets.includes(num.charAt(indexNumber)) || num.charAt(indexNumber) === '' || indexNumber > num.length - 1) {
                                formattedValue += '';
                                break;
                            }
                            else {
                                formattedValue += num.charAt(indexNumber);
                                indexNumber = indexNumber + 1;
                            }
                            indexFormat = indexFormat + ('[[' + brackets + ']]').length - 1;
                        }
                        else if (this.template.charAt(indexFormat) !== 'd') {
                            if (num.charAt(indexNumber) !== '' || this.template.charAt(indexFormat) === '+') {
                                formattedValue += this.template.charAt(indexFormat);
                            }
                        }
                        else {
                            if (num.charAt(indexNumber) === '' || indexNumber > num.length - 1) {
                                formattedValue += '';
                            }
                            else {
                                formattedValue += num.charAt(indexNumber);
                                indexNumber = indexNumber + 1;
                            }
                        }
                    }
                }
                break;
            default:
                if (this.pattern) {
                    if (!(new RegExp(this.pattern).test(value))) {
                        value = this.value;
                    }
                }
                if (this.template) {
                    if (this.cursorPosition === value.length) {
                        cursorEnd = true;
                    }
                    if (this.backspace) {
                        for (indexFormat = 0, indexNumber = 0; indexFormat < this.template.length; indexFormat = indexFormat + 1) {
                            subLength = 0;
                            if (this.template.charAt(indexFormat) === '[' && this.template.charAt(indexFormat + 1) === '[') {
                                brackets = this.template.substr(indexFormat + 2);
                                brackets = brackets.split(/]]/).shift();
                                if (!(new RegExp(`[${brackets}]`).test(value.charAt(indexNumber))) ||
                                    value.charAt(indexNumber) === '' || indexNumber > value.length - 1) {
                                    break;
                                }
                                else {
                                    indexNumber = indexNumber + 1;
                                }
                                indexFormat = indexFormat + ('[[' + brackets + ']]').length - 1;
                            }
                            else {
                                subLength += 1;
                                if (indexNumber === value.length - 1) {
                                    value = (String(value).substring(0, value.length - subLength - 1));
                                    break;
                                }
                                if (this.template.charAt(indexFormat) === value.charAt(indexNumber)) {
                                    indexNumber = indexNumber + 1;
                                }
                            }
                        }
                    }
                    formattedValue = '';
                    for (indexFormat = 0, indexNumber = 0; indexFormat < this.template.length; indexFormat = indexFormat + 1) {
                        if (this.template.charAt(indexFormat) === '[' && this.template.charAt(indexFormat + 1) === '[') {
                            brackets = this.template.substr(indexFormat + 2);
                            brackets = brackets.split(/]]/).shift();
                            if (!new RegExp(`[${brackets}]`).test(value.charAt(indexNumber)) ||
                                value.charAt(indexNumber) === '' || indexNumber > value.length - 1) {
                                formattedValue += '';
                                break;
                            }
                            else {
                                formattedValue += value.charAt(indexNumber);
                                indexNumber = indexNumber + 1;
                            }
                            indexFormat = indexFormat + ('[[' + brackets + ']]').length - 1;
                        }
                        else {
                            if (this.template.charAt(indexFormat) === value.charAt(indexNumber)) {
                                formattedValue += value.charAt(indexNumber);
                                indexNumber = indexNumber + 1;
                            }
                            else {
                                formattedValue += this.template.charAt(indexFormat);
                            }
                        }
                    }
                }
                break;
        }
        if (this.template && value !== formattedValue) {
            value = formattedValue;
            if (cursorEnd) {
                this.cursorPosition = value.length;
            }
            // @ts-ignore
            if (document.selection) {
                // @ts-ignore
                selection = document.selection.createRange();
                selection.moveStart('character', -value.length);
                selection.moveStart('character', this.cursorPosition);
                selection.moveEnd('character', 0);
                selection.select();
            }
            else if (this.element.selectionStart || this.element.selectionStart === 0) {
                this.element.selectionStart = this.cursorPosition;
                this.element.selectionEnd = this.cursorPosition;
            }
        }
        this.model.control.setValue(this.prefixValue
            + ([null, undefined].includes(value) ? '' : value)
            + this.suffixValue, { emitEvent: false });
        this.backspace = false;
        return value;
    }
    updateModel(originalValue, viewValue) {
        let value = viewValue;
        switch (this.typeValue) {
            case 'number':
                if (value !== undefined && value.match(/^\s*[-+]?(?:\d+|\d+\.\d+)$/g) && !value.match(/^\s*[-+]?(?:\d+\.\d*0)$/g)) {
                    value = value !== '' ? parseFloat(value) : undefined;
                    value = isNaN(value) ? undefined : value;
                }
                break;
            case 'tel':
                value = (/^\s*\+/.test(value) ? '+' : '') + value.replace(/\D/g, '');
                break;
        }
        this.model.control.setValue(value, { emitViewToModelChange: false, emitModelToViewChange: false });
        return value;
    }
    forceCursor($event) {
        if (($event === null || $event === void 0 ? void 0 : $event.type) === 'select') {
            $event.preventDefault();
        }
        this.cursorPosition = this.element.selectionStart;
        let cursorPositionEnd = this.element.selectionEnd;
        this.cursorPosition = Math.max(this.prefixValue.length, this.cursorPosition);
        this.cursorPosition = Math.min(this.element.value.length - this.suffixValue.length, this.cursorPosition);
        cursorPositionEnd = Math.max(this.prefixValue.length, cursorPositionEnd);
        cursorPositionEnd = Math.min(this.element.value.length - this.suffixValue.length, cursorPositionEnd);
        this.element.setSelectionRange(this.cursorPosition, cursorPositionEnd, this.element.selectionDirection);
    }
    evalValue($event) {
        var _a;
        if (($event.type === 'keydown' && $event.key === 'Enter') || $event.type === 'blur') {
            switch (this.typeValue) {
                case 'number':
                    const value = this.value;
                    try {
                        // tslint:disable-next-line:no-eval
                        this.updateValue(eval(this.value));
                    }
                    catch (e) {
                        this.updateValue((_a = (this.value + '' || '').match(/(?:[-+]?\d+\.?\d*)/)) === null || _a === void 0 ? void 0 : _a.join(''));
                    }
                    if (+value !== +this.value) {
                        if ($event.type === 'keydown') {
                            $event.stopPropagation();
                            $event.preventDefault();
                        }
                    }
                    break;
            }
        }
    }
    onKeyDown($event) {
        this.backspace = $event.key === 'Backspace';
        this.forceCursor($event);
        this.evalValue($event);
        if (this.backspace && this.cursorPosition <= this.prefixValue.length) {
            $event.preventDefault();
            this.backspace = false;
            return;
        }
        if ($event.key === 'Delete' && this.cursorPosition >= this.element.value.length - this.suffixValue.length) {
            $event.preventDefault();
            return;
        }
        switch (this.typeValue) {
            case 'number':
                if (['ArrowUp', 'ArrowDown'].includes($event.key)) {
                    $event.preventDefault();
                    $event.stopPropagation();
                    $event.stopImmediatePropagation();
                    let multiplier = 1;
                    if ($event.altKey) {
                        multiplier = 0.1;
                    }
                    else {
                        multiplier = $event.metaKey || $event.ctrlKey ? multiplier * 10 : multiplier;
                        multiplier = $event.shiftKey ? multiplier * 100 : multiplier;
                    }
                    if ($event.key === 'ArrowUp') {
                        this.updateValue((((this.value || 0) * 10) + (multiplier * 10)) / 10);
                    }
                    if ($event.key === 'ArrowDown') {
                        this.updateValue((((this.value || 0) * 10) - (multiplier * 10)) / 10);
                    }
                    $event.preventDefault();
                    this.cursorPosition = this.prefixValue.length;
                    const cursorPositionEnd = this.element.value.length - this.suffixValue.length;
                    this.element.setSelectionRange(this.cursorPosition, cursorPositionEnd, 'none');
                }
                break;
        }
    }
    initializeControls() {
        var _a, _b;
        if (this.incrementElement && this.decrementElement) {
            return;
        }
        (_a = this.incrementElement) === null || _a === void 0 ? void 0 : _a.remove();
        (_b = this.decrementElement) === null || _b === void 0 ? void 0 : _b.remove();
        this.incrementElement = this.renderer.createElement('div');
        this.decrementElement = this.renderer.createElement('div');
        this.incrementElement.className = this.decrementElement.className = 'input-group-append';
        this.element.parentElement.append(this.decrementElement, this.incrementElement);
        const incrementButton = this.renderer.createElement('button');
        incrementButton.innerHTML = `<i class="fas fa-sm fa-plus fa-fw"></i>`;
        this.incrementElement.appendChild(incrementButton);
        const decrementButton = this.renderer.createElement('button');
        decrementButton.innerHTML = `<i class="fas fa-sm fa-minus fa-fw"></i>`;
        this.decrementElement.appendChild(decrementButton);
        incrementButton.className = decrementButton.className = 'btn btn-sm btn-primary';
        incrementButton.type = decrementButton.type = 'button';
        pressAndHoldEventInit(incrementButton);
        pressAndHoldEventInit(decrementButton);
        incrementButton.addEventListener('pressandhold', () => this.updateValue((((this.value || 0) * 10) + 10) / 10));
        decrementButton.addEventListener('pressandhold', () => this.updateValue((((this.value || 0) * 10) - 10) / 10));
    }
    ngAfterContentInit() {
        this.element = this.elementRef.nativeElement;
        this.element.autocomplete = 'off';
        this.element.type = 'text';
        if (!this.element.classList.contains('form-control')) {
            this.element.classList.add('form-control');
        }
        if (!this.element.parentElement.classList.contains('input-group')) {
            const parent = this.element.parentNode;
            const inputGroup = this.renderer.createElement('div');
            this.renderer.addClass(inputGroup, 'input-group');
            this.renderer.appendChild(inputGroup, this.element);
            this.renderer.appendChild(parent, inputGroup);
        }
        this.controls = this.controlsValue;
        this.model.valueChanges
            .pipe(takeUntil(this.ngDestroy$), distinctUntilChanged(), map(value => this.updateValue(value)))
            .subscribe();
    }
    ngOnDestroy() {
        this.ngDestroy$.next();
    }
};
AdvanceInputDirective.ctorParameters = () => [
    { type: NgModel, decorators: [{ type: Self }] },
    { type: ElementRef },
    { type: Renderer2 }
];
__decorate([
    Attribute('type'), Input()
], AdvanceInputDirective.prototype, "type", null);
__decorate([
    Attribute('prefix'), Input()
], AdvanceInputDirective.prototype, "prefix", null);
__decorate([
    Attribute('suffix'), Input()
], AdvanceInputDirective.prototype, "suffix", null);
__decorate([
    Attribute('controls'), Input()
], AdvanceInputDirective.prototype, "controls", null);
__decorate([
    Attribute('min'), Input()
], AdvanceInputDirective.prototype, "min", null);
__decorate([
    Attribute('max'), Input()
], AdvanceInputDirective.prototype, "max", null);
__decorate([
    Attribute('template'), Input()
], AdvanceInputDirective.prototype, "template", null);
__decorate([
    Attribute('pattern'), Input()
], AdvanceInputDirective.prototype, "pattern", null);
__decorate([
    HostListener('focus'),
    HostListener('click'),
    HostListener('select', ['$event'])
], AdvanceInputDirective.prototype, "forceCursor", null);
__decorate([
    HostListener('blur', ['$event'])
], AdvanceInputDirective.prototype, "evalValue", null);
__decorate([
    HostListener('keydown', ['$event'])
], AdvanceInputDirective.prototype, "onKeyDown", null);
AdvanceInputDirective = __decorate([
    Directive({
        // tslint:disable-next-line:directive-selector
        selector: 'input[advanceInput]',
    }),
    __param(0, Self())
], AdvanceInputDirective);

let TriggerOnViewportService = class TriggerOnViewportService {
    constructor() {
        this.elements = [];
        merge(fromEvent(window, 'scroll'), fromEvent(window, 'resize'))
            .pipe(debounceTime(1000)).subscribe(() => {
            this.elements.forEach((element, i) => this.processElement(element, i));
        });
    }
    addElement(element) {
        if (this.elements.find(e => e.element.isSameNode(element.element))) {
            return;
        }
        this.elements.push(element);
        window.dispatchEvent(new Event('resize'));
    }
    processElement(element, i) {
        const rect = element.element.getBoundingClientRect();
        if (rect.top >= 0
            && rect.top <= window.innerHeight
            && rect.left >= 0
            && rect.left <= window.innerWidth) {
            element.callback();
            this.elements.splice(i, 1);
        }
    }
};
TriggerOnViewportService.ɵprov = ɵɵdefineInjectable({ factory: function TriggerOnViewportService_Factory() { return new TriggerOnViewportService(); }, token: TriggerOnViewportService, providedIn: "root" });
TriggerOnViewportService = __decorate([
    Injectable({
        providedIn: 'root'
    })
], TriggerOnViewportService);

const IMAGE_SERVICE_CONFIG = new InjectionToken('ImgServiceConfig');
let ImgService = class ImgService {
    constructor(imgService) {
        Object.assign(this, imgService);
    }
};
ImgService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [IMAGE_SERVICE_CONFIG,] }] }
];
ImgService.ɵprov = ɵɵdefineInjectable({ factory: function ImgService_Factory() { return new ImgService(ɵɵinject(IMAGE_SERVICE_CONFIG)); }, token: ImgService, providedIn: "any" });
ImgService = __decorate([
    Injectable({
        providedIn: 'any'
    }),
    __param(0, Inject(IMAGE_SERVICE_CONFIG))
], ImgService);

let ImgDirective = class ImgDirective {
    constructor(elementRef, triggerOnViewportService, imgService) {
        this.elementRef = elementRef;
        this.triggerOnViewportService = triggerOnViewportService;
        this.imgService = imgService;
        this.loadingStrategy = 'lazy';
        this.thumbnailSrcValue = this.imgService.loadingImage;
        this.errorSrcValue = this.imgService.imageNotFound;
    }
    set thumbnailSrc(src) {
        this.thumbnailSrcValue = src;
        if (this.status === 'loading') {
            this.src = src;
        }
    }
    get thumbnailSrc() {
        return this.thumbnailSrcValue;
    }
    set errorSrc(src) {
        this.errorSrcValue = src;
        if (this.status === 'error') {
            this.src = src;
        }
    }
    get errorSrc() {
        return this.errorSrcValue;
    }
    set source(src) {
        this.originalSrc = src;
        this.image = new Image();
        this.image.remove();
        this.image.onload = () => {
            this.status = 'loaded';
            this.src = this.originalSrc;
            this.image.remove();
        };
        this.image.onerror = () => {
            this.status = 'error';
            this.src = (this.errorSrc || this.thumbnailSrc);
            this.image.remove();
        };
        console.log(this.thumbnailSrc);
        this.status = 'loading';
        if (this.thumbnailSrc) {
            this.src = this.thumbnailSrc;
        }
        if (this.loadingStrategy === 'eager') {
            this.image.src = src;
        }
        else if (this.loadingStrategy === 'lazy') {
            this.triggerOnViewportService.addElement({
                element: this.elementRef.nativeElement, callback: () => this.image.src = this.originalSrc
            });
        }
    }
};
ImgDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: TriggerOnViewportService },
    { type: ImgService }
];
__decorate([
    HostBinding('src')
], ImgDirective.prototype, "src", void 0);
__decorate([
    Attribute('loadingStrategy'), Input()
], ImgDirective.prototype, "loadingStrategy", void 0);
__decorate([
    Attribute('thumbnailSrc'), Input('thumbnailSrc')
], ImgDirective.prototype, "thumbnailSrc", null);
__decorate([
    Attribute('errorSrc'), Input('errorSrc')
], ImgDirective.prototype, "errorSrc", null);
__decorate([
    Attribute('src'), Input('src')
], ImgDirective.prototype, "source", null);
ImgDirective = __decorate([
    Directive({
        // tslint:disable-next-line:directive-selector
        selector: 'img'
    })
], ImgDirective);

let LoadingDirective = class LoadingDirective {
    constructor(templateRef, viewContainerRef, renderer) {
        this.templateRef = templateRef;
        this.viewContainerRef = viewContainerRef;
        this.renderer = renderer;
        this.context = {};
    }
    set loading(context) {
        if (context) {
            this.viewContainerRef.clear();
            if (this.loadingElement) {
                this.renderer.removeChild(this.loadingElement.parentNode, this.loadingElement);
            }
            this.context.$implicit = this.context.loading = context;
            this.viewContainerRef.createEmbeddedView(this.templateRef, this.context);
        }
        else {
            this.loadingElement = this.renderer.createElement(`div`);
            this.loadingElement.innerHTML = `<span class="sr-only">Loading...</span>`;
            this.loadingElement.className = 'spinner-grow text-primary m-auto';
            const elem = this.viewContainerRef.element.nativeElement;
            this.renderer.insertBefore(elem.parentNode, this.loadingElement, elem);
        }
    }
};
LoadingDirective.ctorParameters = () => [
    { type: TemplateRef },
    { type: ViewContainerRef },
    { type: Renderer2 }
];
__decorate([
    Input()
], LoadingDirective.prototype, "loading", null);
LoadingDirective = __decorate([
    Directive({
        // tslint:disable-next-line:directive-selector
        selector: '[loading]'
    })
], LoadingDirective);

const NG_MODEL_ERROR_MAPPER = new InjectionToken('NgModelErrorMapper');
// @dynamic
let NgModelErrorMessageDirective = class NgModelErrorMessageDirective {
    constructor(control, errorMapper, renderer, el) {
        this.control = control;
        this.errorMapper = errorMapper;
        this.renderer = renderer;
        this.el = el;
        this.destroyed$ = new Subject();
    }
    ngOnInit() {
        this.parentElement = this.el.nativeElement.parentElement;
        this.smallElement = this.renderer.createElement('small');
        this.renderer.addClass(this.smallElement, 'text-danger');
        if (this.parentElement.classList.contains('input-group')) {
            this.parentElement = this.parentElement.parentElement;
        }
        this.renderer.appendChild(this.parentElement, this.smallElement);
        // this.errors$
        this.control.statusChanges
            .pipe(map(() => this.control.errors), takeUntil(this.destroyed$), debounceTime(200), switchMap(errors => this.errorMapper ? this.errorMapper(errors) : of(errors)), map(errors => {
            if (!errors) {
                return '';
            }
            if (Array.isArray(errors)) {
                return errors;
            }
            else if (typeof errors === 'object') {
                return JSON.stringify(errors);
            }
            return errors;
        }), distinctUntilChanged(), tap(errors => {
            this.smallElement.innerText = typeof errors === 'object' ? Object.values(errors).join(', ') : errors;
            if (!this.el.nativeElement.classList.contains('form-control')) {
                return;
            }
            if (errors) {
                this.el.nativeElement.classList.add('border-danger');
            }
            else {
                this.el.nativeElement.classList.remove('border-danger');
            }
        }))
            .subscribe();
    }
    ngOnDestroy() {
        this.destroyed$.next();
    }
};
NgModelErrorMessageDirective.ctorParameters = () => [
    { type: NgControl, decorators: [{ type: Self }, { type: Optional }] },
    { type: Function, decorators: [{ type: Inject, args: [NG_MODEL_ERROR_MAPPER,] }] },
    { type: Renderer2 },
    { type: ElementRef }
];
NgModelErrorMessageDirective = __decorate([
    Directive({
        // tslint:disable-next-line:directive-selector
        selector: '[ngModel][ngModelErrorMessage]'
    }),
    __param(0, Self()), __param(0, Optional()),
    __param(1, Inject(NG_MODEL_ERROR_MAPPER))
], NgModelErrorMessageDirective);

var AdvancedSelectComponent_1;
function compareOptionValues(a, b) {
    let aValue;
    let bValue;
    // if (typeof a === 'string' || typeof a === 'number') {
    //   if (typeof b === 'string' || typeof b === 'number' ? b === a : b.value === a) {
    //     return true;
    //   }
    // } else {
    //   if (typeof b === 'string' || typeof b === 'number' ? b === a.value : b.value === a.value) {
    //     return true;
    //   }
    // }
    // return false
    aValue = typeof a === 'string' || typeof a === 'number' ? a : a.value;
    aValue = btoa(unescape(encodeURIComponent(JSON.stringify(aValue))));
    bValue = typeof b === 'string' || typeof b === 'number' ? b : b.value;
    bValue = btoa(unescape(encodeURIComponent(JSON.stringify(bValue))));
    return aValue === bValue;
}
let AdvancedOptionComponent = class AdvancedOptionComponent {
    constructor(parent, elementRef) {
        this.parent = parent;
        this.elementRef = elementRef;
        this.index = null;
        this.class = 'dropdown-item';
        this.ngDestroy$ = new Subject();
    }
    set value(value) {
        this.valueValue = value;
        this.updateState();
    }
    get value() {
        return this.valueValue;
    }
    get label() {
        const label = typeof this.value === 'string' || typeof this.value === 'number' ?
            this.value : this.value.label || this.value.value;
        return `${label}`;
    }
    ngAfterContentInit() {
        this.updateState();
        let mousedown = false;
        fromEvent(this.elementRef.nativeElement, 'mousedown')
            .pipe(takeUntil(this.ngDestroy$), filter((event) => !mousedown && event.button === 0))
            .subscribe(() => {
            mousedown = true;
            this.selected = this.parent.selectValue(this.value);
        });
        fromEvent(window, 'mouseup', { capture: true })
            .pipe(takeUntil(this.ngDestroy$), filter((event) => mousedown))
            .subscribe(() => {
            mousedown = false;
        });
        this.parent.highlightedOptionIndex$
            .pipe(takeUntil(this.ngDestroy$))
            .subscribe(index => {
            if (this.index === index) {
                this.highlighted = true;
                this.elementRef.nativeElement.scrollIntoView();
            }
            else {
                this.highlighted = false;
            }
        });
    }
    updateState() {
        const index = this.parent.advancedOptionComponents.findIndex(option => compareOptionValues(this.value, option.value));
        if (index > -1) {
            this.parent.advancedOptionComponents[index] = this;
            this.index = index;
        }
        else {
            this.index = this.parent.advancedOptionComponents.length;
            this.parent.advancedOptionComponents.push(this);
        }
        this.selected = !!this.parent.valueAsArray.find(value => compareOptionValues(this.value, value));
        // this.elementRef.nativeElement.innerHTML = this.label;
    }
    ngOnDestroy() {
        this.ngDestroy$.next();
        // this.parent.advancedOptionComponents.splice(this.index, 1);
    }
};
AdvancedOptionComponent.ctorParameters = () => [
    { type: AdvancedSelectComponent, decorators: [{ type: Host }] },
    { type: ElementRef }
];
__decorate([
    HostBinding('class')
], AdvancedOptionComponent.prototype, "class", void 0);
__decorate([
    Input()
], AdvancedOptionComponent.prototype, "value", null);
__decorate([
    HostBinding('class.highlighted')
], AdvancedOptionComponent.prototype, "highlighted", void 0);
__decorate([
    HostBinding('class.active'), HostBinding('class.selected')
], AdvancedOptionComponent.prototype, "selected", void 0);
AdvancedOptionComponent = __decorate([
    Component({
        // tslint:disable-next-line:component-selector
        selector: 'advanced-option',
        template: `<ng-content></ng-content>`,
        styles: [`
      :host {
        cursor: pointer;
        position: relative;
        word-break: break-word;
      }

      * {
        position: relative;
        z-index: 0;
      }

      :host::before {
        position: absolute;
        content: '';
        top: 0;
        left: 0;
        /*width: 100%;*/
        /*height: 100%;*/
        z-index: 1;
      }

      :host::before ~ * {
        position: relative;
        z-index: 0;
      }

      :host.selected:before {
        content: '\\f14a';
        font-family: 'Font Awesome 5 Free';
        padding: 0.3em;
      }

      :host.highlighted {
        z-index: 30;
        transform-origin: center center;
        /*background-color: #eee;*/
        box-shadow: 0 0 10px 5px rgba(0, 0, 0, 0.2);
      }

    `]
    }),
    __param(0, Host())
], AdvancedOptionComponent);
let AdvancedSelectComponent = AdvancedSelectComponent_1 = class AdvancedSelectComponent {
    constructor(elementRef) {
        this.elementRef = elementRef;
        this.ngDestroy$ = new Subject();
        this.advancedOptionComponents = [];
        this.highlightedOptionIndex$ = new BehaviorSubject(-1);
        this.queryEnabled = false;
        this.arrow = true;
        this.bootstrap = true;
        this.queryChange = new EventEmitter();
        this.uniqueValues = true;
        this.multipleValue = false;
        this.valueChange = new EventEmitter();
        this.onChange = (value) => null;
    }
    set multiple(multiple) {
        if (this.multipleValue === multiple) {
            return;
        }
        this.multipleValue = multiple;
        if (this.value) {
            if (multiple && !Array.isArray(this.value)) {
                this.value = [this.value];
            }
            else if (Array.isArray(this.value)) {
                this.value = this.value.shift();
            }
        }
        else if (multiple) {
            this.value = [];
        }
    }
    get multiple() {
        return this.multipleValue;
    }
    set query(query) {
        if (query) {
            this.highlightedOptionIndex$.next(0);
        }
        this.queryValue = query;
        this.queryChange.emit(query);
    }
    get query() {
        return this.queryValue;
    }
    set value(value) {
        if (value) {
            if (this.multiple) {
                if (Array.isArray(value)) {
                    this.valueValue = value;
                }
                else {
                    this.valueValue = [value];
                }
            }
            else {
                if (Array.isArray(value)) {
                    this.valueValue = value.shift();
                }
                else {
                    this.valueValue = value;
                }
            }
        }
        else {
            this.valueValue = this.multiple ? [] : undefined;
        }
        this.updateModel();
        this.valueChange.emit(this.value);
    }
    get value() {
        return this.valueValue;
    }
    get valueAsArray() {
        if (!this.value) {
            return [];
        }
        return Array.isArray(this.value) ? this.value : [this.value];
    }
    get valueAsOne() {
        return Array.isArray(this.value) ? this.value.shift() : this.value;
    }
    selectValue(option) {
        let selected = false;
        if (this.multiple) {
            const index = this.valueAsArray.findIndex(value => compareOptionValues(value, option));
            if (index === -1 || !this.uniqueValues) {
                this.valueAsArray.push(option);
                selected = true;
            }
            else {
                this.valueAsArray.splice(index, 1);
            }
            setTimeout(() => this.inputElement.nativeElement.focus(), 100);
        }
        else {
            selected = true;
            this.value = option;
            this.popover.hide();
        }
        this.updateModel();
        return selected;
    }
    deselectValue(value) {
        if (this.multiple) {
            const index = this.valueAsArray.findIndex(v => compareOptionValues(value, v));
            if (index !== -1) {
                this.valueAsArray.splice(index, 1);
            }
            if (this.popover.popover.show) {
                setTimeout(() => this.inputElement.nativeElement.focus());
            }
        }
        else {
            this.value = undefined;
        }
        const component = this.advancedOptionComponents.find(option => compareOptionValues(value, option.value));
        if (component) {
            component.selected = false;
        }
        this.updateModel();
    }
    updateModel() {
        var _a;
        if (this.multiple) {
            this.onChange(this.valueAsArray.map(v => typeof v === 'string' || typeof v === 'number' ? v : v.value));
        }
        else {
            this.onChange(typeof this.valueAsOne === 'string' || typeof this.valueAsOne === 'number' ? this.valueAsOne : (_a = this.valueAsOne) === null || _a === void 0 ? void 0 : _a.value);
        }
        setTimeout(() => this.popover.popover.updatePopOverPosition(), 50);
    }
    getSelectedLabel(option) {
        const label = typeof option === 'string' || typeof option === 'number' ? option : option.selectedStateLabel || option.label || option.value;
        return `${label}`;
    }
    ngAfterViewInit() {
        fromEvent(window, 'click')
            .pipe(takeUntil(this.ngDestroy$))
            .subscribe((event) => !(this.elementRef.nativeElement.contains(event.target)) &&
            !(this.popover.popover.element.contains(event.target)) ?
            this.popover.hide()
            : null);
        this.optionsComponentList.changes
            .pipe(takeUntil(this.ngDestroy$))
            .subscribe(() => {
            this.advancedOptionComponents = this.optionsComponentList.toArray();
            this.advancedOptionComponents.forEach((component, index) => component.index = index);
            this.highlightedOptionIndex$.next(-1);
        });
    }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched(fn) {
    }
    writeValue(value) {
        // this.value = value;
    }
    ngOnDestroy() {
        this.ngDestroy$.next();
    }
    enter() {
        const option = this.advancedOptionComponents[this.highlightedOptionIndex$.value];
        if (option) {
            this.advancedOptionComponents[this.highlightedOptionIndex$.value].selected =
                this.selectValue(this.advancedOptionComponents[this.highlightedOptionIndex$.value].value);
            this.inputElement.nativeElement.blur();
        }
    }
    backspace() {
        if (this.query) {
            return true;
        }
        const value = this.valueAsArray[this.valueAsArray.length - 1];
        this.deselectValue(value);
    }
    moveIndex(amount) {
        // if (this.advancedOptionComponents[this.highlightedOptionIndex$]) {
        //   this.advancedOptionComponents[this.highlightedOptionIndex$].highlighted = false;
        // }
        this.highlightedOptionIndex$.next(Math.min(this.advancedOptionComponents.length - 1, Math.max(-1, this.highlightedOptionIndex$.value + amount)));
        // const component = this.advancedOptionComponents[this.highlightedOptionIndex$];
        // if (component) {
        //   component.highlighted = true;
        //   component.elementRef.nativeElement.scrollIntoView();
        // }
        return;
    }
    onFocus() {
        this.popover.show();
        setTimeout(() => {
            if (window.innerHeight <= 400) {
                this.inputElement.nativeElement.scrollIntoView({ behavior: 'smooth' });
            }
        }, 600);
    }
    eventHandle(event) {
        event.preventDefault();
    }
    onArrowClick() {
        if (this.popover.popover.show) {
            this.popover.hide();
        }
        else {
            this.popover.show();
        }
    }
};
AdvancedSelectComponent.ctorParameters = () => [
    { type: ElementRef }
];
__decorate([
    ContentChildren(AdvancedOptionComponent)
], AdvancedSelectComponent.prototype, "optionsComponentList", void 0);
__decorate([
    Input()
], AdvancedSelectComponent.prototype, "multiple", null);
__decorate([
    Input()
], AdvancedSelectComponent.prototype, "queryEnabled", void 0);
__decorate([
    Input()
], AdvancedSelectComponent.prototype, "arrow", void 0);
__decorate([
    Input()
], AdvancedSelectComponent.prototype, "query", null);
__decorate([
    Input()
], AdvancedSelectComponent.prototype, "value", null);
__decorate([
    HostBinding('class.input-group'), HostBinding('class.form-control')
], AdvancedSelectComponent.prototype, "bootstrap", void 0);
__decorate([
    ViewChild('popover')
], AdvancedSelectComponent.prototype, "popover", void 0);
__decorate([
    ViewChild('inputElement')
], AdvancedSelectComponent.prototype, "inputElement", void 0);
__decorate([
    Output()
], AdvancedSelectComponent.prototype, "queryChange", void 0);
__decorate([
    Input()
], AdvancedSelectComponent.prototype, "options", void 0);
__decorate([
    Input()
], AdvancedSelectComponent.prototype, "uniqueValues", void 0);
__decorate([
    Output()
], AdvancedSelectComponent.prototype, "valueChange", void 0);
AdvancedSelectComponent = AdvancedSelectComponent_1 = __decorate([
    Component({
        // tslint:disable-next-line:component-selector
        selector: 'advanced-select',
        template: "<div class=\"advanced-select-popover-target\" popover\n     [popoverContent]=\"popoverTemplate\"\n     popoverMatchHeight=\"true\" popoverMatchWidth=\"true\" #popover=\"popover\"\n     popoverClass=\"advanced-select-popover\"\n     [popoverPlacement]=\"{vertical: 'auto', horizontal: 'center'}\" [popoverArrow]=\"false\"></div>\n<div class=\"input-group-prepend\" *ngIf=\"valueAsArray.length\" style=\"max-width: 100%;\">\n  <span class=\"input-group-text border-0\" style=\"background: transparent;max-width: 100%;\">\n    <div class=\"values-container\">\n      <label class=\"badge bg-primary\"\n             *ngFor=\"let option of valueAsArray\"\n             [innerHTML]=\"getSelectedLabel(option)\" (click)=\"deselectValue(option)\"></label>\n    </div>\n  </span>\n</div>\n<input style=\"min-width: 150px\" type=\"text\" class=\"form-control border-0 rounded\"\n       [(ngModel)]=\"query\"\n       [readOnly]=\"!queryEnabled\"\n       [ngModelOptions]=\"{standalone: true}\"\n       (focus)=\"onFocus()\" #inputElement\n       (keydown.escape)=\"inputElement.blur();popover.hide(); eventHandle($event)\"\n       (keydown.tab)=\"inputElement.blur();popover.hide();\"\n       (keydown.backspace)=\"backspace()\" (keydown.enter)=\"enter(); eventHandle($event)\"\n       (keydown.arrowUp)=\"moveIndex(-1); eventHandle($event)\" (keydown.arrowDown)=\"moveIndex(1); eventHandle($event)\">\n<div class=\"input-group-append\" *ngIf=\"arrow\">\n  <label class=\"input-group-text border-0\"\n         (click)=\"onArrowClick()\"\n         style=\"background: transparent; cursor:pointer;\">\n    <i class=\"fas fa-chevron-{{popover?.popover?.show ? 'up' : 'down'}} fa-fw\"></i>\n  </label>\n</div>\n<ng-template #popoverTemplate>\n  <ng-container *ngIf=\"options && options.length; else content\">\n    <advanced-option *ngFor=\"let option of options\" [value]=\"option\"></advanced-option>\n  </ng-container>\n</ng-template>\n<ng-template #content>\n  <ng-content></ng-content>\n</ng-template>\n",
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => AdvancedSelectComponent_1),
                multi: true
            }
        ],
        styles: ["@charset \"UTF-8\";:host{word-break:break-word;padding:0;position:relative}:host:focus-within{box-shadow:0 0 0 .2rem rgba(13,110,253,.25)}:host:focus-within input:focus{box-shadow:none}:host .badge{cursor:pointer;font-size:1em;margin:.25em}:host .badge::before{content:\"\uF057\u00A0\u00A0\";font-family:\"Font Awesome 5 Free\"}:host input{background:0 0}:host input[readonly]{cursor:pointer}:host .advanced-select-popover-target{position:absolute;width:100%;height:100%;top:0;left:0;z-index:-1}:host .input-group-prepend span{padding:0!important}:host .values-container{max-width:100%;word-break:break-all;white-space:initial;justify-content:flex-start;display:flex;flex-wrap:wrap}"]
    })
], AdvancedSelectComponent);

var ImgModule_1, NgModelErrorMessageModule_1;
let ImageInputComponentsModule = class ImageInputComponentsModule {
};
ImageInputComponentsModule = __decorate([
    NgModule({
        declarations: [ImageInputComponent, ImageGalleryComponent],
        imports: [
            CommonModule,
        ],
        exports: [ImageInputComponent, ImageGalleryComponent],
    })
], ImageInputComponentsModule);
let PopoverModule = class PopoverModule {
};
PopoverModule = __decorate([
    NgModule({
        declarations: [PopoverDirective],
        imports: [CommonModule],
        exports: [PopoverDirective],
    })
], PopoverModule);
// @dynamic
let ImgModule = ImgModule_1 = class ImgModule {
    static forRoot(options) {
        return ({
            ngModule: ImgModule_1,
            providers: [
                {
                    provide: IMAGE_SERVICE_CONFIG,
                    useFactory: (sanitizer) => {
                        const imgService = {
                            imageNotFound: '',
                            loadingImage: '',
                        };
                        if (typeof (options === null || options === void 0 ? void 0 : options.defaultErrorImage) === 'string') {
                            imgService.imageNotFound = sanitizer.bypassSecurityTrustUrl(options.defaultErrorImage);
                        }
                        else if (options === null || options === void 0 ? void 0 : options.defaultErrorImage) {
                            imgService.imageNotFound = sanitizer.bypassSecurityTrustUrl(`data:image/svg+xml,%3C%3Fxml version='1.0' encoding='utf-8'%3F%3E%3C!-- Generator: Adobe Illustrator 24.0.0, SVG Export Plug-In . SVG Version: 6.00 Build 0) --%3E%3Csvg version='1.1' id='Layer_1' focusable='false' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink' x='0px' y='0px' viewBox='0 0 512 512' style='enable-background:new 0 0 512 512;' xml:space='preserve'%3E%3Cstyle type='text/css'%3E .st0%7Bfill:%23AFAFAF;%7D%0A%3C/style%3E%3Cg%3E%3Cpath class='st0' d='M46.3,448l48-48H54c-3.3,0-6-2.7-6-6V118c0-3.3,2.7-6,6-6h328.3l48-48H48C21.5,64,0,85.5,0,112v288 C0,425.9,20.6,447.1,46.3,448z'/%3E%3Cpolygon class='st0' points='416,352 416,272 347.3,203.3 198.7,352 '/%3E%3Cpath class='st0' d='M482.8,67.8L438.7,112H458c3.3,0,6,2.7,6,6v276c0,3.3-2.7,6-6,6H150.7l-48,48H464c26.5,0,48-21.5,48-48V112 C512,92.2,500,75.2,482.8,67.8z'/%3E%3Ccircle class='st0' cx='128' cy='192' r='40'/%3E%3Cpath class='st0' d='M135.5,264.5L96,304v48h46.3l48.9-48.9l-38.7-38.7C147.8,259.8,140.2,259.8,135.5,264.5z'/%3E%3Cg%3E%3Cpath class='st0' d='M56.1,480.4c-3.6,0-7.1-1.4-9.9-4.1c-5.5-5.5-5.5-14.3,0-19.7L460,42.7c5.5-5.5,14.3-5.5,19.7,0 c5.5,5.5,5.5,14.3,0,19.7L66,476.3C63.2,479,59.7,480.4,56.1,480.4z'/%3E%3C/g%3E%3C/g%3E%3C/svg%3E%0A`);
                        }
                        if (typeof (options === null || options === void 0 ? void 0 : options.defaultThumbnailImage) === 'string') {
                            imgService.loadingImage = sanitizer.bypassSecurityTrustUrl(options.defaultThumbnailImage);
                        }
                        else if (options === null || options === void 0 ? void 0 : options.defaultThumbnailImage) {
                            imgService.loadingImage = sanitizer.bypassSecurityTrustUrl(`data:image/svg+xml,%3C%3Fxml version='1.0' encoding='utf-8'%3F%3E%3C!-- Generator: Adobe Illustrator 24.0.0, SVG Export Plug-In . SVG Version: 6.00 Build 0) --%3E%3Csvg version='1.1' id='Layer_1' focusable='false' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink' x='0px' y='0px' viewBox='0 0 512 512' style='enable-background:new 0 0 512 512;' xml:space='preserve'%3E%3Cstyle type='text/css'%3E .st0%7Bfill:%23AFAFAF;%7D%0A%3C/style%3E%3Ccircle class='st0' cx='256' cy='256' r='256'%3E%3Canimate attributeName='r' values='0;256' dur='1s' repeatCount='indefinite'/%3E%3Canimate attributeName='opacity' values='1;0' dur='1s' repeatCount='indefinite'/%3E%3C/circle%3E%3C/svg%3E%0A`);
                        }
                        return imgService;
                    },
                    deps: [DomSanitizer],
                }
            ],
        });
    }
};
ImgModule = ImgModule_1 = __decorate([
    NgModule({
        declarations: [ImgDirective],
        imports: [CommonModule],
        exports: [ImgDirective],
    })
], ImgModule);
let AdvanceInputModule = class AdvanceInputModule {
};
AdvanceInputModule = __decorate([
    NgModule({
        declarations: [AdvanceInputDirective],
        imports: [CommonModule],
        exports: [AdvanceInputDirective],
    })
], AdvanceInputModule);
let LoadingModule = class LoadingModule {
};
LoadingModule = __decorate([
    NgModule({
        declarations: [LoadingDirective],
        imports: [CommonModule],
        exports: [LoadingDirective],
    })
], LoadingModule);
// @dynamic
let NgModelErrorMessageModule = NgModelErrorMessageModule_1 = class NgModelErrorMessageModule {
    static forRoot({ errorMapper }) {
        return {
            ngModule: NgModelErrorMessageModule_1,
            providers: [
                { provide: NG_MODEL_ERROR_MAPPER, useValue: errorMapper }
            ]
        };
    }
};
NgModelErrorMessageModule = NgModelErrorMessageModule_1 = __decorate([
    NgModule({
        declarations: [NgModelErrorMessageDirective],
        imports: [CommonModule],
        exports: [NgModelErrorMessageDirective],
    })
], NgModelErrorMessageModule);
let NgFormQueryParamsModule = class NgFormQueryParamsModule {
};
NgFormQueryParamsModule = __decorate([
    NgModule({
        declarations: [NgFormQueryParamsDirective],
        imports: [CommonModule, FormsModule, RouterModule],
        exports: [NgFormQueryParamsDirective],
    })
], NgFormQueryParamsModule);
let BreadcrumbsModule = class BreadcrumbsModule {
};
BreadcrumbsModule = __decorate([
    NgModule({
        declarations: [BreadcrumbsComponent],
        imports: [CommonModule, RouterModule],
        exports: [BreadcrumbsComponent],
    })
], BreadcrumbsModule);
let PaginationModule = class PaginationModule {
};
PaginationModule = __decorate([
    NgModule({
        declarations: [PaginationComponent],
        imports: [CommonModule, RouterModule],
        exports: [PaginationComponent],
    })
], PaginationModule);
let AdvancedSelectModule = class AdvancedSelectModule {
};
AdvancedSelectModule = __decorate([
    NgModule({
        declarations: [AdvancedSelectComponent, AdvancedOptionComponent],
        imports: [PopoverModule, FormsModule, CommonModule],
        exports: [AdvancedSelectComponent, AdvancedOptionComponent],
    })
], AdvancedSelectModule);
let NgBootstrapModule = class NgBootstrapModule {
};
NgBootstrapModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            ImageInputComponentsModule,
            AdvanceInputModule,
            PopoverModule,
            BreadcrumbsModule,
            PaginationModule,
            ImgModule,
            NgFormQueryParamsModule,
            LoadingModule,
        ],
        exports: [
            ImageInputComponentsModule,
            AdvanceInputModule,
            PopoverModule,
            BreadcrumbsModule,
            PaginationModule,
            ImgModule,
            NgFormQueryParamsModule,
            LoadingModule,
        ]
    })
], NgBootstrapModule);

/*
 * Public API Surface of nabbous-bootstrap-components
 */

/**
 * Generated bundle index. Do not edit.
 */

export { AdvanceInputDirective, AdvanceInputModule, AdvancedSelectModule, BreadcrumbsComponent, BreadcrumbsModule, BreadcrumbsService, GalleryImageValue, IMAGE_SERVICE_CONFIG, ImageGalleryComponent, ImageInputComponent, ImageInputComponentsModule, ImageInputValue, ImgDirective, ImgModule, ImgService, LoadingDirective, LoadingModule, NG_MODEL_ERROR_MAPPER, NgBootstrapModule, NgFormQueryParamsDirective, NgFormQueryParamsModule, NgModelErrorMessageDirective, NgModelErrorMessageModule, PaginationComponent, PaginationModule, Popover, PopoverDirective, PopoverModule, PopoverService, TriggerOnViewportService, AdvancedOptionComponent as ɵa, AdvancedSelectComponent as ɵb };
//# sourceMappingURL=monabbous-ng-bootstrap.js.map
